from vect_utils import *
import math

#--------------------------------------------------------------------
# Exemples de surfaces
#--------------------------------------------------------------------

# Sphère de centre O et de rayon R

def F_sphere(u, v, R=1):
    x = R * math.cos(u) * math.cos(v)
    y = R * math.cos(u) * math.sin(v)
    z = R * math.sin(u)
    return [x, y, z]

bornes_sphere = [- math.pi / 2, math.pi / 2, 0, 2 * math.pi]

# Tore de rayons R, r

def F_tore(u, v, R=3, r=1):
    x = (R + r * math.cos(u)) * math.cos(v)
    y = (R + r * math.cos(u)) * math.sin(v)
    z = r * math.sin(u)
    return [x, y, z]

bornes_tore = [0, 2 * math.pi, 0, 2 * math.pi]

# Hyperboloïde de révolution à 1 nappe

def F_hyper(u, v):
    r = math.sqrt(1 + u ** 2)
    x = r * math.cos(v)
    y = r * math.sin(v)
    z = u
    return [x, y, z]

bornes_hyper = [-3, 3, 0, 2 * math.pi]

# Gaussienne

F_exp = lambda u, v: [u, v, 2 * math.exp(-u ** 2 - v ** 2)]
bornes_exp = [-2, 2, -2, 2]

# Sinus amorti

def F_sin(u, v):
    r = math.sqrt(u ** 2 + v ** 2)
    if r == 0: return (u, v, 3)
    else:
        return [u, v, 3 * math.sin(r) / r]

bornes_sin = [-10, 10, -10, 10]

# -------------------------------------------------------------------
# Dérivées partielles
# -------------------------------------------------------------------

def diffu(F, u, v, bornes, h=1e-5):
    if u + h <= bornes[1]:
        return mul(1 / h, sub(F(u + h, v), F(u, v)))
    else:
        return mul(1 / h, sub(F(u, v), F(u - h, v)))

def diffv(F, u, v, bornes, h=1e-5):
    if v + h <= bornes[3]:
        return mul(1 / h, sub(F(u, v + h), F(u, v)))
    else:
        return mul(1 / h, sub(F(u, v), F(u, v - h)))

# -------------------------------------------------------------------
# Discrétisation
# -------------------------------------------------------------------

def subdi(a, b, n):
    d = (b - a) / n
    return [a + k * d for k in range(n + 1)]

def discretiser(F, bornes, nu, nv):
    umin, umax, vmin, vmax = bornes
    us = subdi(umin, umax, nu)
    vs = subdi(vmin, vmax, nv)
    m = [[F(u,v) for v in vs] for u in us]
    return m

# -------------------------------------------------------------------
# Projection
# -------------------------------------------------------------------

# Base adaptée à un observateur Obs

def base_adaptee(Obs):
    e3 = normaliser(vecteur((0, 0, 0), Obs))
    u = prod_vect((0, 0, 1), e3)
    if u == (0, 0, 0): e1 = (0, 1, 0)
    else: e1 = normaliser(u)
    e2 = prod_vect(e3, e1)
    return [e1, e2, e3]

# Projection du point M vu par l'observateur Obs dans la base adaptée base

def projeter_point(M, Obs, base):
    OObs = vecteur((0, 0, 0), Obs)
    ObsM = vecteur(Obs, M)
    t = - prod_scal(OObs, base[2]) / prod_scal(ObsM, base[2])
    OQ = add(OObs, mul(t, ObsM))
    x = prod_scal(OQ, base[0])
    y = prod_scal(OQ, base[1])
    return (x, y)
