# ---------------------------------------------------------
# Fonctions vectorielles
#
# Marc Lorenzi
# 22 novembre 2020
# ---------------------------------------------------------

# Les points et les vecteurs de l'espace sont représentés par des triplets de flottants.

import math

# Vecteur d'origine A, d'extrémité B

def vecteur(A, B):
    return (B[0] - A[0], B[1] - A[1], B[2] - A[2])

# Somme des vecteurs u et v

def add(u, v):
    a, b, c = u
    x, y, z = v
    return (a + x, b + y, c + z)

# Différence des vecteurs u et v

def sub(u, v):
    a, b, c = u
    x, y, z = v
    return (a - x, b - y, c - z)

# Produit du vecteur u par le réel t

def mul(t, u):
    x, y, z = u
    return (t * x, t * y, t * z)

# Produit scalaire des vecteurs u et v

def prod_scal(u, v):
    return u[0] * v[0] + u[1] * v[1] + u[2] * v[2]

# Norme au carré du vecteur u

def norme2(u):
    return prod_scal(u, u)

# Norme du vecteur u

def norme(u):
    return math.sqrt(norme2(u))

# Vecteur unitaire colinéaire à u et de même sens

def normaliser(u):
    x, y, z = u
    r = math.sqrt(norme2(u))
    if r != 0:
        return (x / r, y / r, z / r)
    else:
        return (0, 0, 0)

# Produit vectoriel des vecteurs u et v

def prod_vect(u, v):
    a1, b1, c1 = u
    a2, b2, c2 = v
    x = b1 * c2 - b2 * c1
    y = c1 * a2 - c2 * a1
    z = a1 * b2 - a2 * b1
    return (x, y, z)
